/*============================================================================
 * User functions for input of calculation parameters.
 *============================================================================*/

/* Code_Saturne version 5.2.0 */

/*
  This file is part of Code_Saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2018 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>
#include <string.h>

#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/*----------------------------------------------------------------------------
 * PLE library headers
 *----------------------------------------------------------------------------*/

#include <ple_coupling.h>

/*----------------------------------------------------------------------------
 *  Local headers
 *----------------------------------------------------------------------------*/

#include "bft_mem.h"
#include "bft_error.h"
#include "bft_printf.h"

#include "cs_base.h"
#include "cs_convection_diffusion.h"
#include "cs_ctwr.h"
#include "cs_fan.h"
#include "cs_field.h"
#include "cs_field_pointer.h"
#include "cs_field_operator.h"
#include "cs_gui_util.h"
#include "cs_grid.h"
#include "cs_internal_coupling.h"
#include "cs_math.h"
#include "cs_mesh.h"
#include "cs_mesh_location.h"
#include "cs_mesh_quantities.h"
#include "cs_halo.h"
#include "cs_halo_perio.h"
#include "cs_log.h"
#include "cs_multigrid.h"
#include "cs_parameters.h"
#include "cs_param_cdo.h"
#include "cs_physical_constants.h"
#include "cs_post.h"
#include "cs_post_util.h"
#include "cs_prototypes.h"
#include "cs_rotation.h"
#include "cs_sles.h"
#include "cs_sles_it.h"
#include "cs_thermal_model.h"
#include "cs_time_moment.h"
#include "cs_time_step.h"
#include "cs_turbomachinery.h"
#include "cs_turbulence_model.h"
#include "cs_selector.h"
#include "cs_rad_transfer.h"

/*----------------------------------------------------------------------------
 *  Header for the current file
 *----------------------------------------------------------------------------*/

#include "cs_prototypes.h"

/*----------------------------------------------------------------------------*/

BEGIN_C_DECLS

/*----------------------------------------------------------------------------*/
/*!
 * \file cs_user_parameters.c
 *
 * \brief User functions for input of calculation parameters.
 *
 * See \subpage parameters for examples.
 */
/*----------------------------------------------------------------------------*/

/*============================================================================
 * User function definitions
 *============================================================================*/

/*----------------------------------------------------------------------------*/
/*!
 * \brief Select physical model options, including user fields.
 *
 * This function is called at the earliest stages of the data setup,
 * so field ids are not available yet.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_model(void)
{

}

/*----------------------------------------------------------------------------*/
/*!
 * \brief Define or modify general numerical and physical user parameters.
 *
 * At the calling point of this function, most model-related most variables
 * and other fields have been defined, so specific settings related to those
 * fields may be set here.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_parameters(void)
{
cs_glob_mesh_quantities_flag |= CS_BAD_CELLS_REGULARISATION; 
cs_glob_mesh_quantities_flag |= CS_CELL_CENTER_CORRECTION;
cs_glob_mesh_quantities_flag |= CS_CELL_FACE_CENTER_CORRECTION;
cs_glob_mesh_quantities_flag |= CS_FACE_RECONSTRUCTION_CLIP;
cs_glob_mesh_quantities_flag |= CS_BAD_CELLS_WARPED_CORRECTION;

/* IN THE LINE BELOW I ATTEMPT TO FLAG MY TEMPERATURE AS BUOYANT*/
/*cs_field_define_key_int("is_buoyant", 1, CS_F_(t)->id);*/
}

/*----------------------------------------------------------------------------*/
/*!
 * \brief  - Specify the elements such as properties, advection fields,
 *           user-defined equations and modules which have been previously
 *           added.
 *
 * \param[in, out]   domain    pointer to a cs_domain_t structure
*/
/*----------------------------------------------------------------------------*/

void
cs_user_cdo_finalize_setup(cs_domain_t   *domain)
{
  CS_UNUSED(domain);
}

/*----------------------------------------------------------------------------*/
/*!
 * \brief Define or modify output user parameters.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_output(void)
{

}

/*----------------------------------------------------------------------------*/
/*!
 * \brief Define linear solver options.
 *
 * This function is called at the setup stage, once user and most model-based
 * fields are defined.
 *
 * Available native iterative linear solvers include conjugate gradient,
 * Jacobi, BiCGStab, BiCGStab2, and GMRES. For symmetric linear systems,
 * an algebraic multigrid solver is available (and recommended).
 *
 * External solvers may also be setup using this function, the cs_sles_t
 * mechanism alowing such through user-define functions.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_linear_solvers(void)
{

}

/*----------------------------------------------------------------------------*/
/*!
 * \brief Define time moments.
 *
 * This function is called at the setup stage, once user and most model-based
 * fields are defined, and before fine control of field output options
 * is defined.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_time_moments(void)
{
/* CALCULATION OF TIME MOMENTS FOR BUOYANCY LES COMPUTATION CASE */
    /* MOMENT < U > */ 
    {
    int moment_f_id[] = {CS_F_(u)->id};
    int moment_c_id[] = {0};
    int n_fields = 1;
    cs_time_moment_define_by_field_ids("U_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }

    /* MOMENT < V > */
    {
    int moment_f_id[] = {CS_F_(u)->id};
    int moment_c_id[] = {1};
    int n_fields = 1;
    cs_time_moment_define_by_field_ids("V_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }
     
    /* MOMENT < W > */
    {
    int moment_f_id[] = {CS_F_(u)->id};
    int moment_c_id[] = {2};
    int n_fields = 1;
    cs_time_moment_define_by_field_ids("W_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }
     
    /* MOMENT < rho > */
    {
    int moment_f_id[] = {CS_F_(rho)->id};
    int moment_c_id[] = {-1};
    int n_fields = 1;
    cs_time_moment_define_by_field_ids("Density_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }
    
    /* MOMENT <turbvisc>*/  
    {
    int moment_f_id[] = {CS_F_(mu_t)->id};
    int moment_c_id[] = {-1};
    int n_fields = 1;
    cs_time_moment_define_by_field_ids("Turbvisc_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }
   
    /* MOMENT < pressure > */
    {
    int moment_f_id[] = {CS_F_(p)->id};
    int moment_c_id[] = {-1};
    int n_fields = 1;
    cs_time_moment_define_by_field_ids("Pressure_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }

    /* MOMENT < temperature > */
    {
    int moment_f_id[] = {CS_F_(t)->id};
    int moment_c_id[] = {-1};
    int n_fields = 1;
    cs_time_moment_define_by_field_ids("Temp_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }
 
    /* MOMENT < boundary_temperature > */
    {
    int moment_f_id[] = {CS_F_(t_b)->id};
    int moment_c_id[] = {-1};
    int n_fields = 1;
    cs_time_moment_define_by_field_ids("Temp_bound_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }
     
    /* MOMENT < u u > */
    {
    int moment_f_id[] = {CS_F_(u)->id, CS_F_(u)->id};
    int moment_c_id[] = {0,0};
    int n_fields = 2;
    cs_time_moment_define_by_field_ids("UU_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    } 

    /* MOMENT < v v > */
    {
    int moment_f_id[] = {CS_F_(u)->id, CS_F_(u)->id};
    int moment_c_id[] = {1, 1};
    int n_fields = 2;
    cs_time_moment_define_by_field_ids("VV_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }

    /* MOMENT < w w > */
    {
    int moment_f_id[] = {CS_F_(u)->id, CS_F_(u)->id};
    int moment_c_id[] = {2,2};
    int n_fields = 2;
    cs_time_moment_define_by_field_ids("WW_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }
    
    /* MOMENT < u v > */
    {
    int moment_f_id[] = {CS_F_(u)->id, CS_F_(u)->id};
    int moment_c_id[] = {0,1};
    int n_fields = 2;
    cs_time_moment_define_by_field_ids("UV_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }
     
    /* MOMENT < u w > */
    {
    int moment_f_id[] = {CS_F_(u)->id, CS_F_(u)->id};
    int moment_c_id[] = {0,2};
    int n_fields = 2;
    cs_time_moment_define_by_field_ids("UW_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    } 
    /* MOMENT < v w > */
    {
    int moment_f_id[] = {CS_F_(u)->id, CS_F_(u)->id};
    int moment_c_id[] = {1,2};
    int n_fields = 2;
    cs_time_moment_define_by_field_ids("VW_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }
    
    /* MOMENT < t u > */
    {
    int moment_f_id[] = {CS_F_(t)->id, CS_F_(u)->id};
    int moment_c_id[] = {-1, 0};
    int n_fields = 2;
    cs_time_moment_define_by_field_ids("TU_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }

    /* MOMENT < t v > */
    {
    int moment_f_id[] = {CS_F_(t)->id, CS_F_(u)->id};
    int moment_c_id[] = {-1, 1};
    int n_fields = 2;
    cs_time_moment_define_by_field_ids("TV_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }
    
    /* MOMENT < t w > */
    {
    int moment_f_id[] = {CS_F_(t)->id, CS_F_(u)->id};
    int moment_c_id[] = {-1, 2};
    int n_fields = 2;
    cs_time_moment_define_by_field_ids("TW_mean",
                                       n_fields,
                                       moment_f_id,
                                       moment_c_id,
                                       CS_TIME_MOMENT_MEAN,
                                       -1, /* nt_start */
                                       1,   /* t_start */
                                       CS_TIME_MOMENT_RESTART_AUTO,
                                       NULL);
    }
}

/*----------------------------------------------------------------------------*/
/*!
 * \brief Define internal coupling options.
 *
 * Options are usually defined using cs_internal_coupling_add_entity.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_internal_coupling(void)
{

}

/*----------------------------------------------------------------------------*/

END_C_DECLS
