!-------------------------------------------------------------------------------

!                      Code_Saturne version 5.2.0
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2018 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_physical_properties.f90
!>
!> \brief Definition of physical variable laws.
!>
!> See \subpage physical_properties for examples.
!>

!===============================================================================
!> usphyv
!>
!> \brief Definition of physical variable laws.
!>
!> \section Warning
!>
!> It is \b forbidden to modify turbulent viscosity \c visct here
!> (a specific subroutine is dedicated to that: \ref usvist)
!>
!> - icp = 0 must <b> have been specified </b>
!>    in \ref usipsu if we wish to define a variable specific heat
!>    cpro_cp (otherwise: memory overwrite).
!>
!> - the kivisl field integer key (scalar_diffusivity_id)
!>    must <b> have been specified </b>
!>    in \ref usipsu if we wish to define a variable viscosity
!>    \c viscls.
!>
!>
!> \remarks
!>  - This routine is called at the beginning of each time step
!>    Thus, <b> AT THE FIRST TIME STEP </b> (non-restart case), the only
!>    values initialized before this call are those defined
!>      - in the GUI or  \ref usipsu (cs_user_parameters.f90)
!>             - density    (initialized at \c ro0)
!>             - viscosity  (initialized at \c viscl0)
!>      - in the GUI or \ref cs_user_initialization
!>             - calculation variables (initialized at 0 by defaut
!>             or to the value given in the GUI or in \ref cs_user_initialization)
!>
!>  - We may define here variation laws for cell properties, for:
!>     - density:                                    rom    kg/m3
!>     - density at boundary faces:                  romb   kg/m3)
!>     - molecular viscosity:                        cpro_viscl  kg/(m s)
!>     - specific heat:                              cpro_cp     J/(kg degrees)
!>     - diffusivities associated with scalars:      cpro_vscalt kg/(m s)
!>
!> \b Warning: if the scalar is the temperature, cpro_vscalt corresponds
!> to its conductivity (Lambda) in W/(m K)
!>
!>
!> The types of boundary faces at the previous time step are available
!>   (except at the first time step, where arrays \c itypfb and \c itrifb have
!>   not been initialized yet)
!>
!> It is recommended to keep only the minimum necessary in this file
!>   (i.e. remove all unused example code)
!>
!>
!> \section usphyv_cell_id Cells identification
!>
!> Cells may be identified using the \ref getcel subroutine.
!> The syntax of this subroutine is described in the
!> \ref cs_user_boundary_conditions subroutine,
!> but a more thorough description can be found in the user guide.
!
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     mbrom         indicator of filling of romb array
!> \param[in]     dt            time step (per cell)
!_______________________________________________________________________________

subroutine usphyv &
 ( nvar   , nscal  ,                                              &
   mbrom  ,                                                       &
   dt     )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use pointe
use numvar
use optcal
use cstphy
use cstnum
use entsor
use parall
use period
use ppppar
use ppthch
use ppincl
use field
use mesh
use lagran
use cs_c_bindings
use Physical_properties_table
!===============================================================================

implicit none

! ARGUMENTS

integer          nvar   , nscal

integer          mbrom

double precision dt(ncelet)

!!! PHYSICAL PROPERTIES CALCULATED THROUGH A LOOKUP TABLE !!!

! LOCAL VARIABLES 
double precision, dimension(:), pointer :: cvar_scalt
double precision, dimension(:,:), pointer :: dens_var
double precision, dimension(:), pointer :: bfpro_rom
double precision, dimension(:), pointer :: cpro_rom
integer :: ivart, iel, iscal, ifcvsl
integer :: ind_lo
double precision :: xvart
double precision :: grad
double precision :: c_val
double precision :: check_val 
double precision, parameter :: deltaT = 0.1d0

if (iscalt .gt. 0) then
   ivart = isca(iscalt)
   call field_get_val_s(ivarfl(ivart), cvar_scalt)
end if   

call field_get_val_s(icrom, cpro_rom)

!!! ENSURE ARRAY HAS BEEN READ !!!
if (.not. table_dens_read) &
   call READ_DATA()

!!! POINTER TO TARGET !!!
dens_var => density_table 

!!! LOOP OVER ALL THE CELLS !!!
do iel =1, ncel
   xvart = cvar_scalt(iel) !RECOVER TEMPERATURE
   
   ind_lo = int((xvart - dens_var(1,1)) / deltaT)
   !!! gives bottom index and +1 is added since fortran indexing starts at 1 !!!
   ind_lo = ind_lo + 1 
   
   !!! IMPLEMENTING A SIMPLE INTERPOLATION BASED ON y = mx + c !!!
   grad = dens_var(2, ind_lo + 1) - dens_var(2, ind_lo) 
   grad = grad / (dens_var(1, ind_lo + 1) - dens_var(1, ind_lo))
    
   !!! WORKOUT C VAL !!!
   c_val = dens_var(2, ind_lo + 1) - (grad * dens_var(1, ind_lo + 1))
   
   
   check_val = (grad * xvart) + c_val
   !!! CHECK FOR RANDOM CRASH !!! 
   if (check_val .lt. 0d0) then
       write(nfecra, *) "****************************************"
       write(nfecra, *) "DENSITY CALCULATED WRONG                "
       write(nfecra, *) xvart, "--> CELL TEMPERATURE"
       write(nfecra, *) dens_var(1, ind_lo + 1), dens_var(1, ind_lo), "--> TABLE DENSITY VALUES"
       write(nfecra, *) grad, "--> GRADIENT"
       write(nfecra, *) c_val, "--> C_VAL"
       write(nfecra, *) "****************************************"  
        
       !!! KILL COMPUTATION !!! 
       call csexit(1)
   end if  
   !!!! ASSIGN DENSITY VALUE !!!
   cpro_rom(iel) = check_val  
end do
   
return
end subroutine usphyv


